/*
 * config.c - configuration handling
 *
 * Copyright (C) 2020 IOPSYS Software Solutions AB. All rights reserved.
 *
 * See LICENSE and CREDITS file for related information.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#ifndef _GNU_SOURCE
#define _GNU_SOURCE
#endif

#include <json-c/json.h>
#include <libubox/blobmsg.h>
#include <libubox/blobmsg_json.h>
#include <libubox/uloop.h>
#include <libubox/ustream.h>
#include <libubox/utils.h>
#include <libubus.h>
#include <uci.h>

#include "debug.h"
#include "topologyd.h"
#include "config.h"


void topologyd_config_dump(struct topologyd_config *cfg)
{
	dbg("Topologyd config ---------\n");
	dbg("Enabled           : %d\n", cfg->enabled);
	dbg("Depth             : %d\n", cfg->depth);
	dbg("Refresh interval  : %d seconds\n", cfg->refresh_int);
	dbg("Log entries       : %d max\n", cfg->maxlog);

	dbg("---------------------------\n");
}

int topologyd_config_defaults(struct topologyd_config *cfg)
{
	if (!cfg)
		return -1;

	cfg->enabled = true;
	cfg->depth = TOPOLOGY_DEPTH;
	cfg->refresh_int = TOPOLOGY_REFRESH_INT;
	cfg->maxlog = TOPOLOGY_LOG_MAX;

	return 0;
}

static int config_get_section_topology(struct topologyd_config *cfg,
						struct uci_section *s)
{
	enum {
		TOPOLOGY_ATTR_ENABLED,
		TOPOLOGY_ATTR_DEPTH,
		TOPOLOGY_ATTR_INTERVAL,
		TOPOLOGY_ATTR_MAXLOG,
		NUM_TOPOLOGY_ATTRS,
	};
	const struct uci_parse_option opts[] = {
		{ .name = "enabled", .type = UCI_TYPE_STRING },
		{ .name = "depth", .type = UCI_TYPE_STRING },
		{ .name = "interval", .type = UCI_TYPE_STRING },
		{ .name = "maxlog", .type = UCI_TYPE_STRING },
	};
	struct uci_option *tb[NUM_TOPOLOGY_ATTRS];


	uci_parse_section(s, opts, NUM_TOPOLOGY_ATTRS, tb);

	if (tb[TOPOLOGY_ATTR_ENABLED]) {
		const char *val = tb[TOPOLOGY_ATTR_ENABLED]->v.string;

		cfg->enabled = atoi(val) == 1 ? true : false;
	}

	if (tb[TOPOLOGY_ATTR_DEPTH]) {
		const char *val = tb[TOPOLOGY_ATTR_DEPTH]->v.string;
		int v = atoi(val);

		if (v > 0)
			cfg->depth = v;
	}

	if (tb[TOPOLOGY_ATTR_INTERVAL]) {
		const char *val = tb[TOPOLOGY_ATTR_INTERVAL]->v.string;
		int v = atoi(val);

		if (v > 0)
			cfg->refresh_int = v;
	}

	if (tb[TOPOLOGY_ATTR_MAXLOG]) {
		const char *val = tb[TOPOLOGY_ATTR_MAXLOG]->v.string;
		int v = atoi(val);

		if (v > 0)
			cfg->maxlog = v;
	}

	return 0;
}

static int config_get_section_extra(struct topologyd_config *cfg,
						struct uci_section *s)
{
	//TODO

	return 0;
}

int topologyd_config_user(struct topologyd_config *cfg)
{
	struct uci_context *ctx;
	struct uci_package *pkg;
	struct uci_element *e;

	ctx = uci_alloc_context();
	if (!ctx)
		return -1;

	if (uci_load(ctx, TOPOLOGY_CONFIG, &pkg)) {
		uci_free_context(ctx);
		return -1;
	}

	uci_foreach_element(&pkg->sections, e) {
		struct uci_section *s = uci_to_section(e);

		if (!strcmp(s->type, "topology")) {
			config_get_section_topology(cfg, s);
		} else if (!strcmp(s->type, "extra")) {
			config_get_section_extra(cfg, s);
		}
	}

	uci_free_context(ctx);

	topologyd_config_dump(cfg);

	return 0;
}
